==> fastec_3.2.3_43.tar.xz: usr/local/bin/fastec/utilities/csrv.sh <==
#!/bin/bash
set -x

# Catch the SIGTERM signal. See if we're exiting because of a reboot or a shutdown.
function term_trap {
    REBOOT=$(systemctl list-jobs | egrep -q 'reboot.target.*start' && echo "rebooting" || echo "not_rebooting")
    PWROFF=$(systemctl list-jobs | egrep -q 'poweroff.target.*start' && echo "poweroff" || echo "not_poweroff")
    logger "(term_trap) $REBOOT $PWROFF"
}

# Make sure we have a link for the WSS.
# if [ ! -L "/tmp/www" ]; then
#     ln -s /home/swdev/sandbox/web /tmp/www
# fi

# terminator -x tail -f /data/cs.log &

# Setup the environment with values we want/need.
export USER=fascam
export HOME=/home/$USER
export SCRATCH=$HOME/.config/Fastec/scratch

# Before we can load our device drivers, we need to disable ACS on the
# two ports used for P2P DMA transfers on the camera head's switch.
SWITCH_VID="10b5"
SWITCH_DID="8714"
PCI_ACS_REG="f2a"
SWITCH_BUS=$(lspci -d $SWITCH_VID:$SWITCH_DID | cut -d':' -f1 | tail -1)
setpci -s 0000:$SWITCH_BUS:01.0 $PCI_ACS_REG.w=0000
setpci -s 0000:$SWITCH_BUS:02.0 $PCI_ACS_REG.w=0000

# Make sure we can find the libraries.
ldconfig /usr/local/lib

# Remove any "default" Altera drivers.
# Better would be to blacklist the drivers. Refer to https://wiki.archlinux.org/index.php/Kernel_modules#Blacklisting.
rmmod altera_cvp &> /dev/null
rmmod fpga_mgr &> /dev/null

# Remove any old drivers.
rmmod fi_fpga &> /dev/null
rmmod fi_pci &> /dev/null

# Add in our new drivers.
modprobe fi_pci bar_select=2
modprobe fi_fpga

trap term_trap SIGTERM

while true; do
    # Start up our web socket server after a small delay to let the 
    # camera server to start.
    # ( sleep 2; /usr/local/bin/fi_wss &> $SCRATCH/wss.log ) &

    # Start up our camera server.
    /usr/local/bin/camera_server &> $SCRATCH/cs.log 
    exit_loop=$?
    logger "Camera Server exit status: $exit_loop"
    logger "Camera Server: $REBOOT $PWROFF"

    # Are we in the middle of a system shutdown?
    if [ "$PWROFF" = "poweroff" ]; then
	# Make sure we unmount the head's SSD.
	# It may not be mounted, but we need to make sure.
	#
	# NOTE: We are ASSUMING that the first NVME drive is ours.
	umount -q /dev/nvme0n1p1

	# Flush the block devices
	/usr/bin/blockdev --flushbufs /dev/nvme0n1

	# Tell the FPGA to turn off the camera head.
	logger "Power off the head now" 
	echo "w 0x1BC 0x3210ABCD" > /proc/fi_pci/regs
    fi

    # The exit status of 115 ('s') means to shutdown.
    if [ "$exit_loop" = "115" ]; then
	sync; sync

	# Make sure we unmount the head's SSD.
	# It may not be mounted, but we need to make sure.
	#
	# NOTE: We are ASSUMING that the first NVME drive is ours.
	/usr/local/sbin/nvme-automount remove /dev/nvme0n1p1

	# Flush the block devices
	/usr/bin/blockdev --flushbufs /dev/nvme0n1

	# Tell the FPGA to turn off the camera head.
	logger "Power off the head now" 
	echo "w 0x1BC 0x3210ABCD" > /proc/fi_pci/regs
	sleep 1

	# Issue a shutdown request.
	logger "Shutdown now"
	shutdown -h now
    fi

    # An exit status of 82 ('R') measn to reboot
    if [ "$exit_loop" = "82" ]; then
	sync; sync

	# Request a reboot.
	logger "Reboot now"
	shutdown -r now
    fi

    # Kill any WSS instances.
    killall fi_wss

    # Is the camera server asking for a restart?
    # The exit status of 114 ('r') means to reboot. 
    if [ "$exit_loop" != "114" ]; then
	exit 0
    fi
done

==> fastec_3.2.12_60.tar.xz: usr/local/bin/fastec/utilities/csrv.sh <==
#!/bin/bash
# set -x

# Catch the SIGTERM signal. See if we're exiting because of a reboot or a shutdown.
function term_trap {
    REBOOT=$(systemctl list-jobs | egrep -q 'reboot.target.*start' && echo "rebooting" || echo "not_rebooting")
    PWROFF=$(systemctl list-jobs | egrep -q 'poweroff.target.*start' && echo "poweroff" || echo "not_poweroff")
    logger "(term_trap) $REBOOT $PWROFF"
}

# Remove the NVME drive(s) from the system.
function remove_drives {
    # Make sure everything is sync'd.
    sync; sync

    # Find any "/dev" entries.
    find /dev | egrep 'nvme[0-9]n[0-9]$' | while read nvme_dev; do
    	# Find any/all partitions and make sure they are unmounted.
	for part in {0..9}; do
	    if [ -e "$nvme_dev"p"$part" ]; then
		/bin/logger "(remove_drives) Unmount $nvme_dev"p"$part"
		umount -q "$nvme_dev"p"$part"
	    fi
	done

	# Flush the block devices, just to be certain.
    	/bin/logger "(remove_drives) Flushing $nvme_dev"
	/usr/bin/blockdev --flushbufs $nvme_dev
    done

    # Find all the NVME drives.
    find /sys/devices -type d | egrep '/nvme$' | while read nvme_item; do
    	# Substitute the nvme folder name with the remove control item.
        remove_path=${nvme_item/nvme/remove}

	# Issue the remove command.
        /bin/logger "(remove_drives) $remove_path"
        echo 1 > $remove_path
    done
}

# Make sure we have a link for the WSS.
# if [ ! -L "/tmp/www" ]; then
#     ln -s /home/swdev/sandbox/web /tmp/www
# fi

# terminator -x tail -f /data/cs.log &

# Setup the environment with values we want/need.
export USER=fascam
export HOME=/home/$USER
export SCRATCH=$HOME/.config/Fastec/scratch

# Before we can load our device drivers, we need to disable ACS on the
# two ports used for P2P DMA transfers on the camera head's switch.
SWITCH_VID="10b5"
SWITCH_DID="8714"
PCI_ACS_REG="f2a"
SWITCH_BUS=$(lspci -d $SWITCH_VID:$SWITCH_DID | cut -d':' -f1 | tail -1)
setpci -s 0000:$SWITCH_BUS:01.0 $PCI_ACS_REG.w=0000
setpci -s 0000:$SWITCH_BUS:02.0 $PCI_ACS_REG.w=0000

# Make sure we can find the libraries.
ldconfig /usr/local/lib

# Remove any "default" Altera drivers.
# Better would be to blacklist the drivers. Refer to https://wiki.archlinux.org/index.php/Kernel_modules#Blacklisting.
rmmod altera_cvp &> /dev/null
rmmod fpga_mgr &> /dev/null

# Remove any old drivers.
rmmod fi_fpga &> /dev/null
rmmod fi_pci &> /dev/null

# Add in our new drivers.
modprobe fi_pci bar_select=2
modprobe fi_fpga

trap term_trap SIGTERM

while true; do
    # Start up our web socket server after a small delay to let the 
    # camera server to start.
    # ( sleep 2; /usr/local/bin/fi_wss &> $SCRATCH/wss.log ) &

    # Start up our camera server.
    /usr/local/bin/camera_server &> $SCRATCH/cs.log 
    exit_loop=$?
    logger "Camera Server exit status: $exit_loop"
    logger "Camera Server: $REBOOT $PWROFF"

    # Are we in the middle of a system shutdown?
    if [ "$PWROFF" = "poweroff" ]; then
	# Remove the device
	logger "Remove the NVME drive" 
	remove_drives

	# Tell the FPGA to turn off the camera head.
	logger "Power off the head now" 
	echo "w 0x1BC 0x3210ABCD" > /proc/fi_pci/regs
    fi

    # The exit status of 115 ('s') means to shutdown.
    if [ "$exit_loop" = "115" ]; then
	# Remove the device
	logger "Remove the NVME drive" 
	remove_drives

	# Tell the FPGA to turn off the camera head.
	logger "Power off the head now" 
	echo "w 0x1BC 0x3210ABCD" > /proc/fi_pci/regs
	sleep 1

	# Issue a shutdown request.
	logger "Shutdown now"
	shutdown -h now
    fi

    # An exit status of 82 ('R') measn to reboot
    if [ "$exit_loop" = "82" ]; then
	sync; sync

	# Request a reboot.
	logger "Reboot now"
	shutdown -r now
    fi

    # Kill any WSS instances.
    killall fi_wss

    # Is the camera server asking for a restart?
    # The exit status of 114 ('r') means to reboot. 
    if [ "$exit_loop" != "114" ]; then
	exit 0
    fi
done
